 
import * as cloudfront from "aws-cdk-lib/aws-cloudfront";
import * as origins from "aws-cdk-lib/aws-cloudfront-origins";
import * as s3 from "aws-cdk-lib/aws-s3";
import { Construct } from "constructs";
import { NextjsExportS3DynamicRoutingDistributionProps } from "./omitCdkTypes";
/**
 * Deploy a static export Next.js site to Cloudfront and S3 while maintaining the ability to use dynamic routes.
 *
 * Deploys Cloudfront, a Cloudfront Function, an S3 Bucket, and an S3 Deployment.
 *
 * With defaults set, if this construct is removed, all resources will be cleaned up.
 */
export interface NextjsExportS3DynamicRoutingSiteProps {
    /**
     * @default ./.next
     */
    readonly nextBuildDir?: string;
    /**
     * The relative path to the Next.js project.
     *
     * @default ./out
     */
    readonly nextExportPath?: string;
    /**
     * Passthrough props to customize the S3 bucket.
     *
     * @default {
     *   publicReadAccess: false,
     *   blockPublicAccess: s3.BlockPublicAccess.BLOCK_ALL,
     *   autoDeleteObjects: true,
     *   removalPolicy: cdk.RemovalPolicy.DESTROY,
     * }
     */
    readonly bucketProps?: s3.BucketProps;
    /**
     * Passthrough props to customize the Cloudfront distribution.
     *
     * @default Sets up the S3 Origin and Cache Policy.
     */
    readonly distributionProps?: NextjsExportS3DynamicRoutingDistributionProps;
    /**
     * Passthrough props to customize the S3 Origin.
     *
     * @default S3 Origin defaults.
     */
    readonly s3OriginProps?: origins.S3OriginProps;
}
/**
 * Deploy an exported, static site using Next.js.
 * Compatible with a Next 13 project using /pages routing.
 *
 * @see For Static Site Limitations: https://nextjs.org/docs/advanced-features/static-html-export
 *
 * Additional Limitations:
 * - Cloudfront function size is capped at 10KB. This may be exceeded if the amount of pages (page types, not static pages with many static paths) is extremely large, as each represents a JSON object in the code.
 *    - For estimation, if we assume around 1000 characters of natural overhead and average 100 characters per page type, this calculates to about 90 page types (files in the pages folder).
 */
export declare class NextjsExportS3DynamicRoutingSite extends Construct {
    private readonly id;
    private readonly props;
    /**
     * Included for convenience, this cache policy is very similar to Amplify's cache policy,
     * but with a higher maxTtl.
     */
    static readonly RECOMMENDED_CACHE_POLICY: cloudfront.CachePolicyProps;
    readonly viewerRequestCloudfrontFunction: cloudfront.Function;
    readonly s3Bucket: s3.Bucket;
    readonly cloudfrontDistribution: cloudfront.Distribution;
    constructor(scope: Construct, id: string, props: NextjsExportS3DynamicRoutingSiteProps);
    /**
     * Rewrites incoming dynamic URLs to the correct path in the S3 bucket.
     * Otherwise, leaves the request alone.
     *
     * This adds almost zero latency, or is so negligible it cannot be measured.
     *
     * @returns cloudfront.Function
     */
    private createCloudfrontUrlRewriterFunction;
    /**
     * This is in part to reduce the storage of these routes, but it also is massively more CPU efficient
     * than using Regex.
     *
     * Cloudfront functions have an arbitrary time * CPU metric that is hard to grasp, but it starts timing out at about
     * 10 routes with using Regex (as prescribed by Vercel's design).
     * I have yet to see a timeout with this method, even with many routes. We are likely to hit size limits
     * before CPU limits.
     *
     * This could probably support catch-all routes, but I have not found a reason to do so, yet.
     *
     * @see NestedStringObject
     * @see The Cloudfront Function Code dynamic routes logic
     */
    private buildDynamicRoutePages;
    /**
     * Builds and minifies the Cloudfront Function code to rewrite URLs as needed.
     *
     * @param staticRoutePagesStringified Used to map static routes to their S3 path.
     * @param dynamicRoutePagesStringified Used to map dynamic routes to their S3 path.
     * @returns string (minified Cloudfront Function code)
     */
    private buildCloudfrontFunctionCode;
    /**
     * Creates a locked down S3 bucket for the static site that only allows access from the Cloudfront distribution.
     *
     * @returns s3.Bucket
     */
    private createS3Bucket;
    /**
     * Creates a cloudfront distribution with logic to handle typical Next.js functionality, like 404 pages.
     *
     * @returns cloudfront.Distribution
     */
    private createCloudfrontDistribution;
    /**
     * Deploys two separate deployments to S3, with different cache control settings.
     * @returns Two s3deploy.BucketDeployment objects.
     */
    private deploySiteToS3;
}
